from typing import BinaryIO


def decode(data: bytes) -> bytes: ...


def decode_file(input: BinaryIO, output: BinaryIO) -> None: ...


def encode(data: bytes) -> bytes: ...


def encode_file(input: BinaryIO, output: BinaryIO, boolwrite_head: bool = False) -> None: ...



# cython: language_level=3
# cython: cdivision=True
from libc.stdint cimport uint8_t, int32_t

from cpython.object cimport PyObject_HasAttrString
from cpython.bytes cimport PyBytes_Check, PyBytes_AsString, PyBytes_Size

from pybase16384 cimport base16384

cdef uint8_t PyFile_Check(object file):
    if PyObject_HasAttrString(file, "read") and PyObject_HasAttrString(file, "write") and PyObject_HasAttrString(file,
                                                                                                                 "seek"):
        return 1
    return 0

cpdef bytes encode(bytes data):
    cdef base16384.LENDAT *cret = base16384.encode(<const uint8_t *> data, <const int32_t> PyBytes_Size(data))
    ret = <bytes> cret.data[:cret.len]
    base16384.LENDAT_Del(&cret)
    return ret

cpdef bytes decode(bytes data):
    cdef base16384.LENDAT *cret = base16384.decode(<const uint8_t *> data, <const int32_t> PyBytes_Size(data))
    ret = <bytes> cret.data[:cret.len]
    base16384.LENDAT_Del(&cret)
    return ret

cpdef void encode_file(object input,
                       object output,
                       bint write_head = False) with gil:
    if not PyFile_Check(input):
        raise TypeError("input except a file-like object, got %s" % type(input).__name__)
    if not PyFile_Check(output):
        raise TypeError("output except a file-like object, got %s" % type(output).__name__)

    if write_head:
        output.write(b'\xfe\xff')
    while True:
        chunk = input.read(7)
        if not PyBytes_Check(chunk):
            raise TypeError(f"input must be a file-like rb object, got {type(input).__name__}")
        ot = encode(chunk)  # type: bytes
        output.write(ot)
        if PyBytes_Size(chunk) < 7:
            break

cpdef void decode_file(object input,
                       object output) with gil:
    if not PyFile_Check(input):
        raise TypeError("input except a file-like object, got %s" % type(input).__name__)
    if not PyFile_Check(output):
        raise TypeError("output except a file-like object, got %s" % type(output).__name__)

    chunk = input.read(1)  # type: bytes
    if not PyBytes_Check(chunk):
        raise TypeError(f"input must be a file-like rb object, got {type(input).__name__}")
    if chunk == b"\xfe": # 去头
        input.read(1)
    else:
        input.seek(0, 0)  # 回到开头
    while True:
        chunk = input.read(2)  # type: bytes
        if chunk:
            if chunk[0] == 61:  # = stream完了
                output.write(chunk)
                break
            else:
                input.seek(-2, 1)
        else:
            break
        chunk = input.read(8)
        ot = decode(chunk)  # type: bytes
        output.write(ot)
        if PyBytes_Size(chunk) < 8:
            break



# cython: language_level=3
# cython: cdivision=True
from libc.stdint cimport uint8_t, int32_t
cdef extern from "base14.h" nogil:
    ctypedef struct LENDAT:
        uint8_t * data
        int32_t len

    LENDAT * encode(const uint8_t * data, const int32_t len)
    LENDAT * decode(const uint8_t * data, const int32_t len)

cdef extern from * nogil:
    """
void LENDAT_Del(LENDAT** self)
{
    if(*self!=NULL)
    {
        PyMem_Free((*self)->data);
        (*self)->data = NULL;
        PyMem_Free(*self);
        *self = NULL;
    }
}
    """
    void LENDAT_Del(LENDAT** self)



from io import BytesIO
from ._core import *

__version__ = "0.1.0rc1"


def encode_from_string(data: str, write_head: bool = False) -> bytes:
    inp = BytesIO(data.encode())
    out = BytesIO()
    encode_file(inp, out, write_head)
    return out.getvalue()


def decode_from_bytes(data: bytes) -> str:
    inp = BytesIO(data)
    out = BytesIO()
    decode_file(inp, out)
    return out.getvalue().decode()



# -*- coding: utf-8 -*-
import os
import re
import glob
from collections import defaultdict

from Cython.Build import cythonize
from setuptools import Extension, setup, find_packages
from setuptools.command.build_ext import build_ext
import platform

BUILD_ARGS = defaultdict(lambda: ['-O3', '-g0'])

for compiler, args in [
    ('msvc', ['/EHsc', '/DHUNSPELL_STATIC', "/Oi", "/O2", "/Ot"]),
    ('gcc', ['-O3', '-g0'])]:
    BUILD_ARGS[compiler] = args


class build_ext_compiler_check(build_ext):
    def build_extensions(self):
        compiler = self.compiler.compiler_type
        args = BUILD_ARGS[compiler]
        for ext in self.extensions:
            ext.extra_compile_args = args
        super().build_extensions()


if platform.architecture()[0] == "64bit":  # 64位
    CPUBIT = 64
else:
    CPUBIT = 32

if platform.system() == "Windows":
    macro_base = [("__WIN64__", "1"), ("new", "PyMem_Malloc")]
else:
    macro_base = [("new", "PyMem_Malloc")]

extensions = [
    Extension("pybase16384._core", ["pybase16384/_core.pyx"] + glob.glob(f'./base16384/{CPUBIT}/*.c'),
              include_dirs=[f"./base16384", "./pybase16384", f"./base16384/{CPUBIT}"],
              library_dirs=[f"./base16384"],
              define_macros=macro_base + [("CPUBIT32", None)] if CPUBIT == 32 else []
              ),
]


def get_dis():
    with open("README.markdown", "r", encoding="utf-8") as f:
        return f.read()


def get_version() -> str:
    path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "pybase16384", "__init__.py")
    with open(path, "r", encoding="utf-8") as f:
        data = f.read()
    result = re.findall(r"(?<=__version__ = \")\S+(?=\")", data)
    return result[0]


packages = find_packages(exclude=('test', 'tests.*', "test*"))


def main():
    version: str = get_version()

    dis = get_dis()
    setup(
        name="pybase16384",
        version=version,
        url="https://github.com/synodriver/pybase16384",
        packages=packages,
        keywords=["encode", "decode", "16384"],
        description="16384 encode and decode",
        long_description_content_type="text/markdown",
        long_description=dis,
        author="synodriver",
        author_email="diguohuangjiajinweijun@gmail.com",
        python_requires=">=3.6",
        install_requires=["cython"],
        license='GPLv3',
        classifiers=[
            "Development Status :: 4 - Beta",
            "Operating System :: OS Independent",
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
            "Topic :: Multimedia :: Sound/Audio",
            "Programming Language :: C",
            "Programming Language :: Cython",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Programming Language :: Python :: Implementation :: CPython"
        ],
        include_package_data=True,
        zip_safe=False,
        cmdclass={'build_ext': build_ext_compiler_check},
        ext_modules=cythonize(extensions,
                              compiler_directives={"cdivision": True,
                                                   "embedsignature": True,
                                                   "boundscheck": False,
                                                   "wraparound": False}),
    )


if __name__ == "__main__":
    main()
